# itss.pl -- spin overrides for the ITSS web page template. -*- perl -*-
# $Id: its.pl,v 1.9 2006-05-09 18:24:59 eagle Exp $
#
# This file should be loaded with "do" inside spin, by passing spin the -o
# option to load it.  It overrides various default methods inside spin to do
# the right thing with the web page templates.

# We use Text::Template for our output.
use Text::Template ();

# Use Date::Parse to parse CVS Ids into timestamps and strftime to format
# them.
use Date::Parse ();
use POSIX qw(strftime);

# Used to hold page details until we're ready to print it out.
use vars qw(%VARS);

# We're redefining functions here intentionally.
no warnings 'redefine';

##############################################################################
# Basic overrides
##############################################################################

# Override the default output method, doing the same operations as we normally
# do but also accumulating the output into the page body variable until we're
# ready to generate the output.
sub output {
    my $output = join ('', @_);
    if ($SPACE) {
        $output =~ s%^(\s*(?:</(?!body)[^>]+>\s*)*)%$1$SPACE%;
        $SPACE = '';
    }
    if ($output =~ s/\n(\s+)\z/\n/) { $SPACE = $1 }
    $VARS{body} .= $output;
}

##############################################################################
# Supporting functions
##############################################################################

# Munge HTML from an external source so that it's suitable for being added to
# the body of the ITSS web page template output.  This supports the
# conventions of cvs2xhtml, cl2xhtml, and faq2html.
sub munge_html {
    my ($html) = @_;
    $html =~ s/^<\?xml [^>]+>\s*//gm;
    $html =~ s/^<!DOCTYPE [^>]+>\s*//gm;
    $html =~ s/^<html [^>]+>\s*//gm;
    $html =~ s%^<head>.*?</head>\s*%%gsm;
    $html =~ s/^<body>\s*//gm;
    $html =~ s%^<h1>.*?</h1>\s*%%gsm;
    $html =~ s%<h2>(.*?)</h2>%<h3>$1</h3>%gsm;
    $html =~ s%<h3>(.*?)</h3>%<p><strong>$1</strong></p>%gsm;
    $html =~ s%<h4>(.*?)</h4>%<p><em>$1</em></p>%gsm;
    $html =~ s%^</body>\s*%%gm;
    $html =~ s%^</html>\s*%%gm;
    $html =~ s{<p class=\"subheading\">(.*?)</p>}
        {<p><strong>$1</strong></p>}gsm;
    $html =~ s{^<span class=\"file\">(.*?)</span>}
        {<code style="color: teal;">$1</code>}gsm;
    $html =~ s{^<span class=\"function\">(.*?)</span>}
        {<span style="color: green;">$1</span>}gsm;
    $html =~ s%^<span class=\"revision\">(.*?)</span>%<strong>$1</strong>%gsm;
    $html =~ s{^<p class=\"changed\">(.*?)</p>}
        {<p style="font-size: x-small;">$1</p>}gsm;
    $html =~ s{^<span class=\"added\">(.*?)</span>}
        {<span style="color: red;">$1</span>}gsm;
    $html =~ s{^<span class=\"removed\">(.*?)</span>}
        {<span style="color: blue;">$1</span>}gsm;
    return $html;
}

##############################################################################
# New commands
##############################################################################

# This is an amazingly ugly hack.  I hate tables.  \tablehead and \tablerow
# will lift the span into the surrounding element so that it will work.
sub do_colspan {
    my ($format, $count, $value) = @_;
    return (0, qq(<span colspan="$count">) . parse ($value) . '</span>');
}

# Embed the results of an external command into the body of a page.  This
# works exactly like embed, but runs a command and slurps up its output.
sub do_command {
    my ($format, $command) = @_;
    my $file = parse ($command);
    open (HTML, "$command |")
        or warn "$0:$FILE:$.: Can't execute $command: $!\n";
    local $/;
    my $html = <HTML>;
    close HTML;
    return (1, munge_html ($html));
}

# Embed a regular HTML page into the body of a page.  This isn't really
# robust; it's currently designed to only handle the output of faq2html,
# cl2xhtml, and cvs2xhtml.  Takes a file name as its only argument.
sub do_embed {
    my ($format, $file) = @_;
    $file = parse ($file);
    open (HTML, $file) or warn "$0:$FILE:$.: Can't open $file: $!\n";
    local $/;
    my $html = <HTML>;
    close HTML;
    return (1, munge_html ($html));
}

# Store the service or subheading name, used in some of the navigation.  These
# is now ignored since we have only one heading at the top of the page and we
# get it from \heading.
sub do_service { return (1, '') }
sub do_subheading { return (1, '') }

# Store the HelpSU problem category.  For right now, we don't do anything with
# this.
sub do_pcat { return (1, '') }

# Handle a single sidebar link.  The first argument is the URL, or empty if
# this is the page that we're currently on.  The second argument is the target
# of the link.  Currently, the template doesn't do anything special with links
# to the current page.
sub do_sidelink {
    my ($format, $url, $text) = @_;
    $url = parse ($url);
    $text = parse ($text);
    my $output;
    if ($url) {
        $output = qq(<li><a href="$url">$text</a></li>);
    } else {
        $output = qq(<li>$text</li>);
    }
    return (1, $output);
}

# Create a sidebar link section.  The first argument is used for the title of
# the section, and the second argument will contain all of the links (created
# with \sidelink).
sub do_sidebar {
    my ($format, $heading, $links) = @_;
    $heading = parse ($heading);
    $heading =~ s/:\s*$//;
    $links = parse ($links);
    if ($VARS{sidecontent}) {
        $VARS{sidecontent} .= qq(\n);
    }
    $VARS{sidecontent} .= qq(<h1>$heading</h1>\n<ul>$links</ul>);
    return (1, '');
}

# Create a site navigation bar.  The only argument will contain all of the
# links (created with \sidelink since the format is identical).
sub do_sitenav {
    my ($format, $links) = @_;
    $links = parse ($links);
    $VARS{sitenav} = qq(<div id="nav2"><ul>\n$links</ul></div>);
    return (1, '');
}

# Create a crumb navigation section.  The only argument will contain all of
# the links (created with \sidelink since the format is identical).
sub do_crumbnav {
    my ($format, $links) = @_;
    $links = parse ($links);
    $VARS{crumbnav} = qq(<ul>\n$links</ul>);
    return (1, '');
}

##############################################################################
# Command overrides
##############################################################################

# Override h1 to include the right header tags (we actually use h2).  Insert
# an anchor if the format starts with #.  If the format is specified and
# doesn't start with #, set a class for the <h2> tag.
sub do_h1 {
    my ($format, $text) = @_;
    my $output = border;
    $text = parse ($text);
    if ($format && $format =~ /^\#/) {
        my $tag = $format;
        $tag =~ s/^\#//;
        $output .= qq(<a name="$tag"></a>);
    }
    if ($format && $format !~ /^\#/) {
        $class = $format;
        $output .= qq(<h2 class="$class">$text);
    } else {
        $output .= qq(<h2>$text);
    }
    $output =~ s/\n\z//;
    $output .= "</h2>\n";
    return (1, $output);
}

# Downgrade h2 to <h3> since we're lowering the level of all headings.
sub do_h2 {
    my ($format, $text) = @_;
    my $output = border;
    $text = parse ($text);
    if ($format && $format =~ /^\#/) {
        my $tag = $format;
        $tag =~ s/^\#//;
        $text = qq(<a name="$tag">$text</a>);
    }
    $output .= '<h3' . format_string ($format) . '>' . $text;
    $output =~ s/\n\z//;
    $output .= "</h3>\n";
    return (1, $output);
}

# Downgrade h3 to <h4> since we're lowering the level of all headings.
sub do_h3 {
    my ($format, $text) = @_;
    my $output = border;
    $text = parse ($text);
    if ($format && $format =~ /^\#/) {
        my $tag = $format;
        $tag =~ s/^\#//;
        $text = qq(<a name="$tag">$text</a>);
    }
    $output .= '<h4' . format_string ($format) . '>' . $text;
    $output =~ s/\n\z//;
    $output .= "</h4>\n";
    return (1, $output);
}

# Downgrade h4 to <h5> since we're lowering the level of all headings.
sub do_h4 {
    my ($format, $text) = @_;
    my $output = border;
    $text = parse ($text);
    if ($format && $format =~ /^\#/) {
        my $tag = $format;
        $tag =~ s/^\#//;
        $text = qq(<a name="$tag">$text</a>);
    }
    $output .= '<h5' . format_string ($format) . '>' . $text;
    $output =~ s/\n\z//;
    $output .= "</h5>\n";
    return (1, $output);
}

# Since we are downgrading and there is no <h6> defined, lets just italicize h5
sub do_h5 {
    my ($format, $text) = @_;
    my $output = border;
    $text = parse ($text);
    if ($format && $format =~ /^\#/) {
        my $tag = $format;
        $tag =~ s/^\#//;
        $text = qq(<a name="$tag">$text</a>);
    }
    $output .= '<h5><i' . format_string ($format) . '>' . $text;
    $output =~ s/\n\z//;
    $output .= "</i></h5>\n";
    return (1, $output);
}

# Rather than printing anything out, the \heading command should just stash
# the page title in $VARS{title} and in $VARS{heading}.  The second argument
# is which ITSS template to use.
sub do_heading {
    my ($format, $title, $template) = @_;
    $title = parse ($title);
    $VARS{title} = $title;
    $VARS{heading} = $title;
    if ($template) {
        $template .= '.tmpl' unless $template =~ /\./;
        $VARS{template} = $template;
    }
    return (1, '');
}

# \signature does the actual work of printing out the page.
sub do_signature {
    $VARS{body} .= border;
    my $file = $VARS{template} || 'service.tmpl';
    $file = "/usr/share/spin/templates/$file"
        unless $file =~ m%^/%;
    my $template = Text::Template->new (TYPE => 'FILE', SOURCE => $file)
        or warn "$0:$FILE:$.: Can't construct template from $file: "
            . $Text::Template::ERROR;
    my $mtime = (stat($FILES[0][0]))[9];
    $VARS{mtime} = strftime("%Y-%m-%d", localtime($mtime));
    my $output = $template->fill_in (HASH => \%VARS);
    if (defined $output) {
        print OUT $output or warn "$0:$FILE:$.: Can't write output: $!\n";
    } else {
        warn "$0:$FILE:$.: Template failed: $Text::Template::ERROR\n";
    }
    undef %VARS;
    return (1, '');
}

# Override \table to use some default parameters.
sub do_table {
    my ($format, $options, $body) = @_;
    $format ||= 'red-header';
    $options ||= 'cellspacing="1"';
    my $tag = $options ? "table $options" : 'table';
    return block ($tag, '', $format, $body);
}

# Register or re-register our commands.
$commands{colspan}    = [ 2, \&do_colspan    ];
$commands{command}    = [ 1, \&do_command    ];
$commands{crumbnav}   = [ 1, \&do_crumbnav   ];
$commands{embed}      = [ 1, \&do_embed      ];
$commands{h1}         = [ 1, \&do_h1         ];
$commands{h2}         = [ 1, \&do_h2         ];
$commands{h3}         = [ 1, \&do_h3         ];
$commands{h4}         = [ 1, \&do_h4         ];
$commands{h5}         = [ 1, \&do_h5         ];
$commands{heading}    = [ 2, \&do_heading    ];
$commands{pcat}       = [ 1, \&do_pcat       ];
$commands{service}    = [ 1, \&do_service    ];
$commands{signature}  = [ 0, \&do_signature  ];
$commands{sidebar}    = [ 2, \&do_sidebar    ];
$commands{sidelink}   = [ 2, \&do_sidelink   ];
$commands{sitenav}    = [ 1, \&do_sitenav    ];
$commands{subheading} = [ 1, \&do_subheading ];
$commands{table}      = [ 2, \&do_table      ];
